// Example implementation of Read-a-Card API
//
// This code is provided as an example only
// Copyright 2009, Dot ORIGIN. All rights reserved

#include "stdafx.h"
#include "ReadAComm.h"
#include "ReadACommDlg.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#endif

// CReadACommDlg dialog

static UINT s_readAcardMessageID = 0;


CReadACommDlg::CReadACommDlg(CWnd* pParent /*=NULL*/)
	: CDialog(CReadACommDlg::IDD, pParent)
{
	m_hIcon = AfxGetApp()->LoadIcon(IDR_MAINFRAME);
	m_registered = FALSE;
}

void CReadACommDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	DDX_Control(pDX, IDC_REGISTER_BUTTON, m_registerButtonCtrl);
	DDX_Control(pDX, IDC_UNREGISTER_BUTTON, m_unregisterButtonCtrl);
	DDX_Control(pDX, IDC_MESSAGE_WINDOW, m_messageEditCtrl);
}

BEGIN_MESSAGE_MAP(CReadACommDlg, CDialog)
	ON_WM_PAINT()
	ON_WM_QUERYDRAGICON()
	//}}AFX_MSG_MAP
	ON_BN_CLICKED(IDC_REGISTER_BUTTON, &CReadACommDlg::OnBnClickedRegisterButton)
	ON_BN_CLICKED(IDC_UNREGISTER_BUTTON, &CReadACommDlg::OnBnClickedUnregisterButton)
	ON_WM_COPYDATA()
END_MESSAGE_MAP()


// CReadACommDlg message handlers

BOOL CReadACommDlg::OnInitDialog()
{
	CDialog::OnInitDialog();

	// Set the icon for this dialog.  The framework does this automatically
	//  when the application's main window is not a dialog
	SetIcon(m_hIcon, TRUE);			// Set big icon
	SetIcon(m_hIcon, FALSE);		// Set small icon

	s_readAcardMessageID = RegisterWindowMessage(_T("READACARD"));

	UpdateButtons();

	return TRUE;  // return TRUE  unless you set the focus to a control
}

// If you add a minimize button to your dialog, you will need the code below
//  to draw the icon.  For MFC applications using the document/view model,
//  this is automatically done for you by the framework.
void CReadACommDlg::OnPaint()
{
	if (IsIconic())
	{
		CPaintDC dc(this); // device context for painting

		SendMessage(WM_ICONERASEBKGND, reinterpret_cast<WPARAM>(dc.GetSafeHdc()), 0);

		// Center icon in client rectangle
		int cxIcon = GetSystemMetrics(SM_CXICON);
		int cyIcon = GetSystemMetrics(SM_CYICON);
		CRect rect;
		GetClientRect(&rect);
		int x = (rect.Width() - cxIcon + 1) / 2;
		int y = (rect.Height() - cyIcon + 1) / 2;

		// Draw the icon
		dc.DrawIcon(x, y, m_hIcon);
	}
	else
	{
		CDialog::OnPaint();
	}
}

// The system calls this function to obtain the cursor to display while the user drags
//  the minimized window.
HCURSOR CReadACommDlg::OnQueryDragIcon()
{
	return static_cast<HCURSOR>(m_hIcon);
}

bool CReadACommDlg::RegisterWithReadACard(bool enable) {
	bool result = FALSE;
	LRESULT r = 0;

	// Find the Read-a-Card window 
	// The title will either be "Read-a-Card" or "Read-a-Card (TRIAL MODE)"
	HWND readaWnd = ::FindWindow(NULL, _T("Read-a-Card"));

	if (!readaWnd) {
		readaWnd = ::FindWindow(NULL, _T("Read-a-Card (TRIAL MODE)"));
	}

	if (!readaWnd) {
		AfxMessageBox(_T("Failed to find Read-a-Card"));
	}
	else {
		// Send a message to the Read-a-Card window providing our window's handle as the LPARAM.
		// The action mask value of 3 will register with Read-a-Card and disable the keyboard wedge.
		// The action mask value of 0 will un-register with Read-a-Card and enable the keyboard wedge.
		r = ::SendMessage(readaWnd, s_readAcardMessageID, enable ? 3 : 0, (LPARAM) this->m_hWnd);
		
		if (r == ERROR_SUCCESS) {
			result = TRUE;
		}
		else {
			CString message;
			message.Format(_T("Error %s with Read-a-Card (error = %d)"), enable ? _T("registering") : _T("unregistering"), r);
			AfxMessageBox(message);
		}
	}

	return result;
}

void CReadACommDlg::UpdateButtons() {
	m_registerButtonCtrl.EnableWindow(!m_registered);
	m_unregisterButtonCtrl.EnableWindow(m_registered);
}


void CReadACommDlg::OnBnClickedRegisterButton()
{
	if (RegisterWithReadACard(TRUE)) {
		m_registered = TRUE;
		UpdateButtons();
	}
}

void CReadACommDlg::OnBnClickedUnregisterButton()
{
	if (RegisterWithReadACard(FALSE)) {
		m_registered = FALSE;
		UpdateButtons();
	}
}


#define MAX_LOG_DISPLAY_CHARS 10000

void CReadACommDlg::AppendToEdit(HWND hEdit, CString message) {
	int length = ::GetWindowTextLength(hEdit);

	// If the text is getting too long prune it by 1/10 of its maximum size
	if (length > MAX_LOG_DISPLAY_CHARS) {
		::SendMessage(hEdit, EM_SETSEL, 0, MAX_LOG_DISPLAY_CHARS / 10);
		::SendMessage(hEdit, EM_REPLACESEL, FALSE, (LPARAM)_T("...."));
	}

	length = ::GetWindowTextLength(hEdit);

	// Insert the new text at the end of the current text.
	::SendMessage(hEdit, EM_SETSEL, length, length);
	::SendMessage(hEdit, EM_REPLACESEL, FALSE, (LPARAM)(LPCTSTR) message);
	
	// Set the selection at the end of the text
	length = ::GetWindowTextLength(hEdit);
	::SendMessage(hEdit, EM_SETSEL, length, length);
	::SendMessage(hEdit, EM_SCROLLCARET, 0, 0);
}


typedef struct {
	WCHAR cardID[256];
	WCHAR readerSerial[256];
	WCHAR cardType[256];
	WCHAR timeStamp[256];
} READ_A_CARD_DATA;


BOOL CReadACommDlg::OnCopyData(CWnd* pWnd, COPYDATASTRUCT* pCopyDataStruct) 
{
	// Interpret the WM_COPYDATA data pointer as a pointer to a READ_A_CARD_DATA 
	// structure.
	READ_A_CARD_DATA *tcsBuff = (READ_A_CARD_DATA *)(pCopyDataStruct->lpData);

	// Display the four strings
	CString message;
	message.Format(_T("ID = %s, Serial = %s, Type = %s, Time = %s\n"), 
		tcsBuff->cardID, tcsBuff->readerSerial, tcsBuff->cardType, tcsBuff->timeStamp);

	AppendToEdit(m_messageEditCtrl.GetSafeHwnd(), message);

	return CDialog::OnCopyData(pWnd, pCopyDataStruct);
}


